@@ -45,7 +45,7 @@ void febug_start_path(const char * path) {
 	if(getenv("FEBUG_DONT"))
 		return;
 
-	if((febug_global_controlled_socket = socket(AF_UNIX, SOCK_SEQPACKET, 0)) == -1) {
+	if((febug_global_controlled_socket = socket(AF_UNIX, SOCK_SEQPACKET | SOCK_CLOEXEC, 0)) == -1) {
 		fprintf(stderr, "febug_start_path: socket: %s\n", strerror(errno));
 		return;
 	}
@@ -56,7 +56,7 @@ void febug_start_path(const char * path) {
 
 	struct sockaddr_un addr = {0};
 	addr.sun_family         = AF_UNIX;
-	(void)strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
+	strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
 	if(connect(febug_global_controlled_socket, (const struct sockaddr *)&addr, sizeof(addr)) == -1) {
 		fprintf(stderr, "febug_start_path: connect: %s\n", strerror(errno));
 		close(febug_global_controlled_socket);
@@ -90,8 +90,7 @@ void febug_start_path(const char * path) {
 	 */
 	char cmsgbuf[CMSG_SPACE(sizeof(struct cmsgcred))];
 
-	struct msghdr msg;
-	memset(&msg, 0, sizeof(msg));
+	struct msghdr msg = {0};
 	msg.msg_control    = cmsgbuf;
 	msg.msg_controllen = sizeof(cmsgbuf);
 
@@ -135,11 +134,10 @@ void febug_register_type(uint64_t type, void (*formatter)(int, size_t)) {
 	if(febug_global_controlled_socket == -1)
 		return;
 
-	struct formatter_t * new = calloc(1, sizeof(*new));
+	struct formatter_t * new = malloc(sizeof(*new));
 	if(!new)
 		return;
-	new->type      = type;
-	new->formatter = formatter;
+	*new = (struct formatter_t){.type = type, .formatter = formatter};
 
 	if(!tsearch(new, &formatters, formatter_t_compare))
 		free(new);
