@@ -43,11 +43,11 @@ FEBUG_MAYBE(extern) int febug_global_controlled_socket;
 
 FEBUG_MAYBE() void febug_start_path(const char * path);
 
-static void febug_start() {
-	return febug_start_path(FEBUG_SOCKET);
+static inline void febug_start(void) {
+	febug_start_path(FEBUG_SOCKET);
 }
 
-FEBUG_MAYBE() void febug_end();
+FEBUG_MAYBE() void febug_end(void);
 
 
 FEBUG_MAYBE() void febug_register_type(uint64_t type, void (*formatter)(int, size_t));
@@ -56,14 +56,14 @@ FEBUG_MAYBE() void febug_debug_handler(int);
 
 __attribute__((format(printf, 4, 0))) FEBUG_MAYBE() void febug_wrap_signalv(uint64_t type, const void * data, uint8_t signal, const char * name, va_list ap);
 
-__attribute__((format(printf, 4, 5))) static void febug_wrap_signal(uint64_t type, const void * data, uint8_t signal, const char * name, ...) {
+__attribute__((format(printf, 4, 5))) static inline void febug_wrap_signal(uint64_t type, const void * data, uint8_t signal, const char * name, ...) {
 	va_list ap;
 	va_start(ap, name);
 	febug_wrap_signalv(type, data, signal, name, ap);
 	va_end(ap);
 }
 
-__attribute__((format(printf, 3, 4))) static void febug_wrap(uint64_t type, const void * data, const char * name, ...) {
+__attribute__((format(printf, 3, 4))) static inline void febug_wrap(uint64_t type, const void * data, const char * name, ...) {
 	va_list ap;
 	va_start(ap, name);
 	febug_wrap_signalv(type, data, FEBUG_SIGNUM, name, ap);
@@ -74,13 +74,17 @@ FEBUG_MAYBE() void febug_unwrap(const void * data);
 
 
 #if FEBUG_DONT
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-parameter"
+#pragma GCC diagnostic ignored "-Wunused-variable"
 static int febug_global_controlled_socket = -1;
 static void febug_start_path(const char * path) {}
-static void febug_end() {}
+static void febug_end(void) {}
 static void febug_register_type(uint64_t type, void (*formatter)(int, size_t)) {}
 static void febug_debug_handler(int _) {}
 static void febug_wrap_signalv(uint64_t type, const void * data, uint8_t signal, const char * name, va_list ap) {}
 static void febug_unwrap(const void * data) {}
+#pragma GCC diagnostic pop
 #endif
 
 
