@@ -77,6 +77,10 @@ namespace febug {
 				wrapper_impl(signal, name, ap);
 			}
 
+#pragma GCC diagnostic push
+#if defined(__clang__)
+#pragma GCC diagnostic ignored "-Wc++20-designator"
+#endif
 			~wrapper() noexcept {
 #if !FEBUG_DONT
 				if(global_controlled_socket != -1) {
@@ -90,12 +94,15 @@ namespace febug {
 			void wrapper_impl(uint8_t signal, const char * name, va_list ap) noexcept {
 #if !FEBUG_DONT
 				if(global_controlled_socket != -1) {
-					febug_message msg{.variable_id = (uint64_t)this->data, .variable_type = typeid(std::decay_t<T>).hash_code(), .signal = signal};
+					febug_message msg{.variable_id = (uint64_t)this->data, .variable_type = typeid(std::decay_t<T>).hash_code(), .signal = signal, .name = {}};
 					std::vsnprintf(msg.name, sizeof(msg.name), name, ap);
 					send(global_controlled_socket, &msg, sizeof(msg), 0);
 				}
+#else
+			(void)signal, (void)name, (void)ap;
 #endif
 			}
+#pragma GCC diagnostic pop
 		};
 
 
