@@ -24,8 +24,9 @@ typedef __register_t register_t;
 #endif
 
 #include <assert.h>
+#include <cstdio>
 #include <errno.h>
-#include <iostream>
+#include <inttypes.h>
 
 #include <signal.h>
 #include <sys/types.h>
@@ -90,6 +91,8 @@ static int parse_pid(const char * begin, const char * end, pid_t & into) {
 
 #include <febug-abi.h>
 
+using namespace std::literals;
+
 
 static timespec get_now() {
 	timespec ret;
@@ -126,8 +129,9 @@ static std::shared_mutex processes_mtx{};
 static bool debug = false;
 
 
+#define SV(sv) (int)(sv).size(), (sv).data()
 template <class Root, class Process, class Entry>
-static int dispatch_path(const std::string_view path, Root && root_f, Process && process_f, Entry && entry_f) {
+static int dispatch_path(const std::string_view & path, Root && root_f, Process && process_f, Entry && entry_f) {
 	if(path.size() == 0 || path.size() == 1) {
 		return root_f();
 	} else {
@@ -138,25 +142,25 @@ static int dispatch_path(const std::string_view path, Root && root_f, Process &&
 		pid_t val{};
 		if(auto err = parse_pid(first_chunk.begin(), first_chunk.end(), val)) {
 			if(debug)
-				std::cerr << path << ": " << first_chunk << ": " << std::strerror(err) << '\n';
+				std::fprintf(stderr, "%.*s: %.*s: %s\n", SV(path), SV(first_chunk), std::strerror(err));
 			return -ENOENT;
 		}
 
 
 		const std::shared_lock p_lock{processes_mtx};
 		if(const auto process = processes.find(val); process != processes.end()) {
-			if(first_chunk_end_idx == std::string_view::npos) {
+			if(first_chunk_end_idx == std::string_view::npos)
 				return process_f(process);
-			} else {
+			else {
 				auto name_chunk = path.substr(1 + first_chunk_end_idx + 1);
 
 				if(const auto entry =
 				       std::find_if(process->second.entries.begin(), process->second.entries.end(), [&](auto && ent) { return ent.second.name == name_chunk; });
-				   entry != process->second.entries.end()) {
+				   entry != process->second.entries.end())
 					return entry_f(process, entry);
-				} else {
+				else {
 					if(debug)
-						std::cerr << path << ": " << name_chunk << '\n';
+						std::fprintf(stderr, "%.*s: %.*s\n", SV(path), SV(name_chunk));
 					return -ENOENT;
 				}
 			}
@@ -206,253 +210,251 @@ struct read_state {
 };
 
 
-static const fuse_operations fops = [] {
-	fuse_operations ops{};
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
+static const fuse_operations fops{
+    /** Get file attributes.
+     *
+     * Similar to stat().  The 'st_dev' and 'st_blksize' fields are
+     * ignored. The 'st_ino' field is ignored except if the 'use_ino'
+     * mount option is given. In that case it is passed to userspace,
+     * but libfuse and the kernel will still assign a different
+     * inode for internal use (called the "nodeid").
+     *
+     * `fi` will always be NULL if the file is not currently open, but
+     * may also be NULL if the file is open.
+     */
+    .getattr = [](const char * path, struct stat * ret FUSE3_ONLY(, struct fuse_file_info *)) -> int {
+	    return dispatch_path(
+	        path,
+	        [&] {
+		        root_stat(*ret);
+		        return 0;
+	        },
+	        [&](auto && process) {
+		        process_stat(*ret, process->second);
+		        return 0;
+	        },
+	        [&](auto && process, auto && entry) {
+		        entry_stat(*ret, process->second, entry->second);
+		        return 0;
+	        });
+    },
 
-	/** Get file attributes.
-	 *
-	 * Similar to stat().  The 'st_dev' and 'st_blksize' fields are
-	 * ignored. The 'st_ino' field is ignored except if the 'use_ino'
-	 * mount option is given. In that case it is passed to userspace,
-	 * but libfuse and the kernel will still assign a different
-	 * inode for internal use (called the "nodeid").
-	 *
-	 * `fi` will always be NULL if the file is not currently open, but
-	 * may also be NULL if the file is open.
-	 */
-	ops.getattr = [](const char * path, struct stat * ret FUSE3_ONLY(, struct fuse_file_info *)) -> int {
-		return dispatch_path(
-		    path,
-		    [&] {
-			    root_stat(*ret);
-			    return 0;
-		    },
-		    [&](auto && process) {
-			    process_stat(*ret, process->second);
-			    return 0;
-		    },
-		    [&](auto && process, auto && entry) {
-			    entry_stat(*ret, process->second, entry->second);
-			    return 0;
-		    });
-	};
-
-	/** Open a file
-	 *
-	 * Open flags are available in fi->flags. The following rules
-	 * apply.
-	 *
-	 *  - Creation (O_CREAT, O_EXCL, O_NOCTTY) flags will be
-	 *    filtered out / handled by the kernel.
-	 *
-	 *  - Access modes (O_RDONLY, O_WRONLY, O_RDWR) should be used
-	 *    by the filesystem to check if the operation is
-	 *    permitted.  If the ``-o default_permissions`` mount
-	 *    option is given, this check is already done by the
-	 *    kernel before calling open() and may thus be omitted by
-	 *    the filesystem.
-	 *
-	 *  - When writeback caching is enabled, the kernel may send
-	 *    read requests even for files opened with O_WRONLY. The
-	 *    filesystem should be prepared to handle this.
-	 *
-	 *  - When writeback caching is disabled, the filesystem is
-	 *    expected to properly handle the O_APPEND flag and ensure
-	 *    that each write is appending to the end of the file.
-	 *
-	 *  - When writeback caching is enabled, the kernel will
-	 *    handle O_APPEND. However, unless all changes to the file
-	 *    come through the kernel this will not work reliably. The
-	 *    filesystem should thus either ignore the O_APPEND flag
-	 *    (and let the kernel handle it), or return an error
-	 *    (indicating that reliably O_APPEND is not available).
-	 *
-	 * Filesystem may store an arbitrary file handle (pointer,
-	 * index, etc) in fi->fh, and use this in other all other file
-	 * operations (read, write, flush, release, fsync).
-	 *
-	 * Filesystem may also implement stateless file I/O and not store
-	 * anything in fi->fh.
-	 *
-	 * There are also some flags (direct_io, keep_cache) which the
-	 * filesystem may set in fi, to change the way the file is opened.
-	 * See fuse_file_info structure in <fuse_common.h> for more details.
-	 *
-	 * If this request is answered with an error code of ENOSYS
-	 * and FUSE_CAP_NO_OPEN_SUPPORT is set in
-	 * `fuse_conn_info.capable`, this is treated as success and
-	 * future calls to open will also succeed without being send
-	 * to the filesystem process.
-	 *
-	 */
-	ops.open = [](const char * path, struct fuse_file_info * fi) -> int {
-		return dispatch_path(
-		    path, [] { return 0; }, [](auto &&) { return 0; },
-		    [&](auto && process, auto && entry) {
-			    if(fi->flags & O_DIRECTORY)
-				    return -EISDIR;
-			    else if(fi->flags & (O_WRONLY | O_RDWR))
-				    return -EROFS;
+    /** Open a file
+     *
+     * Open flags are available in fi->flags. The following rules
+     * apply.
+     *
+     *  - Creation (O_CREAT, O_EXCL, O_NOCTTY) flags will be
+     *    filtered out / handled by the kernel.
+     *
+     *  - Access modes (O_RDONLY, O_WRONLY, O_RDWR) should be used
+     *    by the filesystem to check if the operation is
+     *    permitted.  If the ``-o default_permissions`` mount
+     *    option is given, this check is already done by the
+     *    kernel before calling open() and may thus be omitted by
+     *    the filesystem.
+     *
+     *  - When writeback caching is enabled, the kernel may send
+     *    read requests even for files opened with O_WRONLY. The
+     *    filesystem should be prepared to handle this.
+     *
+     *  - When writeback caching is disabled, the filesystem is
+     *    expected to properly handle the O_APPEND flag and ensure
+     *    that each write is appending to the end of the file.
+     *
+     *  - When writeback caching is enabled, the kernel will
+     *    handle O_APPEND. However, unless all changes to the file
+     *    come through the kernel this will not work reliably. The
+     *    filesystem should thus either ignore the O_APPEND flag
+     *    (and let the kernel handle it), or return an error
+     *    (indicating that reliably O_APPEND is not available).
+     *
+     * Filesystem may store an arbitrary file handle (pointer,
+     * index, etc) in fi->fh, and use this in other all other file
+     * operations (read, write, flush, release, fsync).
+     *
+     * Filesystem may also implement stateless file I/O and not store
+     * anything in fi->fh.
+     *
+     * There are also some flags (direct_io, keep_cache) which the
+     * filesystem may set in fi, to change the way the file is opened.
+     * See fuse_file_info structure in <fuse_common.h> for more details.
+     *
+     * If this request is answered with an error code of ENOSYS
+     * and FUSE_CAP_NO_OPEN_SUPPORT is set in
+     * `fuse_conn_info.capable`, this is treated as success and
+     * future calls to open will also succeed without being send
+     * to the filesystem process.
+     *
+     */
+    .open = [](const char * path, struct fuse_file_info * fi) -> int {
+	    return dispatch_path(
+	        path, [] { return 0; }, [](auto &&) { return 0; },
+	        [&](auto && process, auto && entry) {
+		        if(fi->flags & O_DIRECTORY)
+			        return -EISDIR;
+		        else if(fi->flags & (O_WRONLY | O_RDWR))
+			        return -EROFS;
 #if __linux__
-			    else if(fi->flags & O_PATH)
-				    return -EOPNOTSUPP;
+		        else if(fi->flags & O_PATH)
+			        return -EOPNOTSUPP;
 #endif
 
-			    int commie_pipe[2];
-			    if(pipe(commie_pipe) == -1)
-				    return -ENOMEM;
+		        int commie_pipe[2];
+		        if(pipe(commie_pipe) == -1)
+			        return -ENOMEM;
 
-			    fi->direct_io = true;
-			    fi->fh        = (std::uint64_t) new read_state{0, commie_pipe[0]};
+		        fi->direct_io = true;
+		        fi->fh        = (std::uint64_t) new read_state{0, commie_pipe[0]};
 
-			    // https://stackoverflow.com/a/15313135/2851815
-			    char cmsgbuf[CMSG_SPACE(sizeof(commie_pipe[1]))];
+		        // https://stackoverflow.com/a/15313135/2851815
+		        char cmsgbuf[CMSG_SPACE(sizeof(commie_pipe[1]))];
 
-			    attn_febug_message afmsg{.variable_id = entry->first, .variable_type = entry->second.type};
-			    iovec afmsg_i{&afmsg, sizeof(afmsg)};
+		        attn_febug_message afmsg{.variable_id = entry->first, .variable_type = entry->second.type};
+		        iovec afmsg_i{&afmsg, sizeof(afmsg)};
 
-			    msghdr msg{};
-			    msg.msg_iov        = &afmsg_i;
-			    msg.msg_iovlen     = 1;
-			    msg.msg_control    = cmsgbuf;
-			    msg.msg_controllen = sizeof(cmsgbuf);  // necessary for CMSG_FIRSTHDR to return the correct value
+		        msghdr msg{};
+		        msg.msg_iov        = &afmsg_i;
+		        msg.msg_iovlen     = 1;
+		        msg.msg_control    = cmsgbuf;
+		        msg.msg_controllen = sizeof(cmsgbuf);  // necessary for CMSG_FIRSTHDR to return the correct value
 
-			    auto cmsg        = CMSG_FIRSTHDR(&msg);
-			    cmsg->cmsg_level = SOL_SOCKET;
-			    cmsg->cmsg_type  = SCM_RIGHTS;
-			    cmsg->cmsg_len   = CMSG_LEN(sizeof(commie_pipe[1]));
-			    memcpy(CMSG_DATA(cmsg), &commie_pipe[1], sizeof(commie_pipe[1]));
-			    msg.msg_controllen = cmsg->cmsg_len;  // total size of all control blocks
+		        auto cmsg        = CMSG_FIRSTHDR(&msg);
+		        cmsg->cmsg_level = SOL_SOCKET;
+		        cmsg->cmsg_type  = SCM_RIGHTS;
+		        cmsg->cmsg_len   = CMSG_LEN(sizeof(commie_pipe[1]));
+		        memcpy(CMSG_DATA(cmsg), &commie_pipe[1], sizeof(commie_pipe[1]));
+		        msg.msg_controllen = cmsg->cmsg_len;  // total size of all control blocks
 
-			    if((sendmsg(process->second.comm_fd, &msg, MSG_NOSIGNAL)) == -1) {
-				    std::cerr << path << ": open: sendmsg: " << std::strerror(errno) << '\n';
-				    return -EBUSY;
-			    }
+		        if((sendmsg(process->second.comm_fd, &msg, MSG_NOSIGNAL)) == -1) {
+			        std::fprintf(stderr, "%s: open: sendmsg: %s\n", path, std::strerror(errno));
+			        return -EBUSY;
+		        }
 
-			    close(commie_pipe[1]);
+		        close(commie_pipe[1]);
 
-			    if(entry->second.signal != SIGKILL)
-				    if(kill(process->first, (int)entry->second.signal) == -1)
-					    std::cerr << path << ": open: kill: " << std::strerror(errno) << '\n';
+		        if(entry->second.signal != SIGKILL)
+			        if(kill(process->first, (int)entry->second.signal) == -1)
+				        std::fprintf(stderr, "%s: open: kill: %s\n", path, std::strerror(errno));
 
-			    return 0;
-		    });
-	};
+		        return 0;
+	        });
+    },
 
-	/** Read data from an open file
-	 *
-	 * Read should return exactly the number of bytes requested except
-	 * on EOF or error, otherwise the rest of the data will be
-	 * substituted with zeroes.	 An exception to this is when the
-	 * 'direct_io' mount option is specified, in which case the return
-	 * value of the read system call will reflect the return value of
-	 * this operation.
-	 */
-	ops.read = [](const char *, char * obuf, size_t obuflen, off_t offset, struct fuse_file_info * fi) -> int {
-		if(fi->fh) {
-			auto state = (read_state *)fi->fh;
+    /** Read data from an open file
+     *
+     * Read should return exactly the number of bytes requested except
+     * on EOF or error, otherwise the rest of the data will be
+     * substituted with zeroes.	 An exception to this is when the
+     * 'direct_io' mount option is specified, in which case the return
+     * value of the read system call will reflect the return value of
+     * this operation.
+     */
+    .read = [](const char *, char * obuf, size_t obuflen, off_t offset, struct fuse_file_info * fi) -> int {
+	    if(fi->fh) {
+		    auto state = (read_state *)fi->fh;
 
-			if(offset != state->offset)
-				return -EBADF;
+		    if(offset != state->offset)
+			    return -EBADF;
 
-			if(auto ret = read(state->read_end, obuf, obuflen); ret != -1) {
-				state->offset += ret;
-				return ret;
-			} else
-				return -errno;
-		} else
-			return -ENOENT;
-	};
+		    if(auto ret = read(state->read_end, obuf, obuflen); ret != -1) {
+			    state->offset += ret;
+			    return ret;
+		    } else
+			    return -errno;
+	    } else
+		    return -ENOENT;
+    },
 
-	/** Release an open file
-	 *
-	 * Release is called when there are no more references to an open
-	 * file: all file descriptors are closed and all memory mappings
-	 * are unmapped.
-	 *
-	 * For every open() call there will be exactly one release() call
-	 * with the same flags and file descriptor.	 It is possible to
-	 * have a file opened more than once, in which case only the last
-	 * release will mean, that no more reads/writes will happen on the
-	 * file.  The return value of release is ignored.
-	 */
-	ops.release = [](const char *, struct fuse_file_info * fi) -> int {
-		if(fi->fh) {
-			auto state = (read_state *)fi->fh;
-			close(state->read_end);
-			delete state;
-			fi->fh = 0;
-		}
+    /** Release an open file
+     *
+     * Release is called when there are no more references to an open
+     * file: all file descriptors are closed and all memory mappings
+     * are unmapped.
+     *
+     * For every open() call there will be exactly one release() call
+     * with the same flags and file descriptor.	 It is possible to
+     * have a file opened more than once, in which case only the last
+     * release will mean, that no more reads/writes will happen on the
+     * file.  The return value of release is ignored.
+     */
+    .release = [](const char *, struct fuse_file_info * fi) -> int {
+	    if(fi->fh) {
+		    auto state = (read_state *)fi->fh;
+		    close(state->read_end);
+		    delete state;
+		    fi->fh = 0;
+	    }
 
-		return 0;
-	};
+	    return 0;
+    },
 
-	/** Open directory
-	 *
-	 * Unless the 'default_permissions' mount option is given,
-	 * this method should check if opendir is permitted for this
-	 * directory. Optionally opendir may also return an arbitrary
-	 * filehandle in the fuse_file_info structure, which will be
-	 * passed to readdir, releasedir and fsyncdir.
-	 *
-	 * default_permissions is always set
-	 */
-	ops.opendir = [](const char * path, struct fuse_file_info *) -> int {
-		return dispatch_path(
-		    path, [] { return 0; }, [](auto &&) { return 0; }, [](auto &&, auto &&) { return -ENOTDIR; });
-	};
+    /** Open directory
+     *
+     * Unless the 'default_permissions' mount option is given,
+     * this method should check if opendir is permitted for this
+     * directory. Optionally opendir may also return an arbitrary
+     * filehandle in the fuse_file_info structure, which will be
+     * passed to readdir, releasedir and fsyncdir.
+     *
+     * default_permissions is always set
+     */
+    .opendir = [](const char * path, struct fuse_file_info *) -> int {
+	    return dispatch_path(
+	        path, [] { return 0; }, [](auto &&) { return 0; }, [](auto &&, auto &&) { return -ENOTDIR; });
+    },
 
-	/** Read directory
-	 *
-	 * The filesystem may choose between two modes of operation:
-	 *
-	 * 1) The readdir implementation ignores the offset parameter, and
-	 * passes zero to the filler function's offset.  The filler
-	 * function will not return '1' (unless an error happens), so the
-	 * whole directory is read in a single readdir operation.
-	 *
-	 * 2) The readdir implementation keeps track of the offsets of the
-	 * directory entries.  It uses the offset parameter and always
-	 * passes non-zero offset to the filler function.  When the buffer
-	 * is full (or an error happens) the filler function will return
-	 * '1'.
-	 */
-	ops.readdir = [](const char * path, void * dest, fuse_fill_dir_t op, off_t, struct fuse_file_info * FUSE3_ONLY(, fuse_readdir_flags)) -> int {
-		return dispatch_path(
-		    path,
-		    [&] {
-			    char buf[20 + 1]{};
-			    struct stat sb;
-			    for(auto && [pid, process] : processes) {
-				    snprintf(buf, sizeof(buf), "%d", pid);
-				    process_stat(sb, process);
+    /** Read directory
+     *
+     * The filesystem may choose between two modes of operation:
+     *
+     * 1) The readdir implementation ignores the offset parameter, and
+     * passes zero to the filler function's offset.  The filler
+     * function will not return '1' (unless an error happens), so the
+     * whole directory is read in a single readdir operation.
+     *
+     * 2) The readdir implementation keeps track of the offsets of the
+     * directory entries.  It uses the offset parameter and always
+     * passes non-zero offset to the filler function.  When the buffer
+     * is full (or an error happens) the filler function will return
+     * '1'.
+     */
+    .readdir = [](const char * path, void * dest, fuse_fill_dir_t op, off_t, struct fuse_file_info * FUSE3_ONLY(, fuse_readdir_flags)) -> int {
+	    return dispatch_path(
+	        path,
+	        [&] {
+		        char buf[20 + 1]{};
+		        struct stat sb;
+		        for(auto && [pid, process] : processes) {
+			        snprintf(buf, sizeof(buf), "%d", pid);
+			        process_stat(sb, process);
 
-				    if(op(dest, buf, &sb, 0 FUSE3_ONLY(, FUSE_FILL_DIR_PLUS)) == 1)
-					    return -EBUSY;
-			    }
+			        if(op(dest, buf, &sb, 0 FUSE3_ONLY(, FUSE_FILL_DIR_PLUS)) == 1)
+				        return -EBUSY;
+		        }
 
-			    return 0;
-		    },
-		    [&](auto && process) {
-			    struct stat sb;
-			    for(auto && [id, entry] : process->second.entries) {
-				    (void)id;
-				    entry_stat(sb, process->second, entry);
+		        return 0;
+	        },
+	        [&](auto && process) {
+		        struct stat sb;
+		        for(auto && [id, entry] : process->second.entries) {
+			        (void)id;
+			        entry_stat(sb, process->second, entry);
 
-				    if(op(dest, entry.name.c_str(), &sb, 0 FUSE3_ONLY(, FUSE_FILL_DIR_PLUS)) == 1)
-					    return -EBUSY;
-			    }
+			        if(op(dest, entry.name.c_str(), &sb, 0 FUSE3_ONLY(, FUSE_FILL_DIR_PLUS)) == 1)
+				        return -EBUSY;
+		        }
 
-			    return 0;
-		    },
-		    [](auto &&, auto &&) {
-			    __builtin_unreachable();
-			    return 0;
-		    });
-	};
-
-	return ops;
-}();
+		        return 0;
+	        },
+	        [](auto &&, auto &&) {
+		        __builtin_unreachable();
+		        return 0;
+	        });
+    }};
+#pragma GCC diagnostic pop
 
 
 #if __linux__
@@ -472,40 +474,31 @@ struct control_socket {
 	int fd            = -1;
 
 	int init() {
-		if((this->fd = socket(AF_UNIX, SOCK_SEQPACKET, 0)) == -1) {
-			std::cerr << "socket: " << std::strerror(errno) << '\n';
-			return 1;
-		}
+		if((this->fd = socket(AF_UNIX, SOCK_SEQPACKET, 0)) == -1)
+			return std::fprintf(stderr, "socket: %s\n", std::strerror(errno)), 1;
 
 		const char * path = DEFAULT_SOCK;
 		if(auto pe = std::getenv("FEBUG_SOCKET"))
 			path = pe;
 
-		if(unlink(path) == -1 && errno != ENOENT) {
-			std::cerr << "unlink(" << path << "): " << std::strerror(errno) << '\n';
-			return 1;
-		}
+		if(unlink(path) == -1 && errno != ENOENT)
+			return std::fprintf(stderr, "unlink(%s): %s\n", path, std::strerror(errno)), 1;
 
 		sockaddr_un addr{};
 		addr.sun_family = AF_UNIX;
 		strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
-		if(bind(this->fd, (const sockaddr *)&addr, sizeof(addr)) == -1) {
-			std::cerr << "bind(" << addr.sun_path << "): " << std::strerror(errno) << '\n';
-			return 1;
-		}
+		if(bind(this->fd, (const sockaddr *)&addr, sizeof(addr)) == -1)
+			return std::fprintf(stderr, "bind(%s): %s\n", addr.sun_path, std::strerror(errno)), 1;
 
 		chmod(path, 0777);
 
 		if(listen(this->fd, 50) == -1) {
-			std::cerr << "listen: " << std::strerror(errno) << '\n';
-			unlink(path);
-			return 1;
+			std::fprintf(stderr, "listen: %s\n", std::strerror(errno));
+			return unlink(path), 1;
 		}
 
-		if(!(this->path = realpath(addr.sun_path, nullptr))) {
-			std::cerr << "realpath(" << addr.sun_path << "): " << std::strerror(errno) << '\n';
-			return 1;
-		}
+		if(!(this->path = realpath(addr.sun_path, nullptr)))
+			return std::fprintf(stderr, "realpath(%s): %s\n", addr.sun_path, std::strerror(errno)), 1;
 
 		return 0;
 	}
@@ -514,7 +507,7 @@ struct control_socket {
 		close(this->fd);
 
 		if(this->path && unlink(this->path) == -1)
-			std::cerr << "unlink(" << this->path << "): " << std::strerror(errno) << '\n';
+			std::fprintf(stderr, "unlink(%s): %s\n", this->path, std::strerror(errno));
 
 		free((void *)this->path);
 		this->path = nullptr;
@@ -565,13 +558,12 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 #endif
 	const auto auth_process = [&](int fd, cmsgcred & process_creds) {
 		if(debug)
-			std::cerr << "fd " << fd << ": is PID " << process_creds.cmcred_pid << " and authed as " << process_creds.cmcred_uid << '/' << process_creds.cmcred_gid
-			          << '\n';
+			std::fprintf(stderr, "fd %d: is PID %d and authed as %d/%d\n", fd, process_creds.cmcred_pid, process_creds.cmcred_uid, process_creds.cmcred_gid);
 
 #if MSG_NOSIGNAL_MISSING
 		int dont = 1;
 		if(fcntl(fd, F_SETNOSIGPIPE, &dont) == -1)
-			std::cerr << "fd " << fd << " (" << process_creds.cmcred_pid << "): fcntl(F_SETNOSIGPIPE, 1): " << std::strerror(errno) << '\n';
+			std::fprintf(stderr, "fd %d (%d): fcntl(F_SETNOSIGPIPE, 1): %s\n", fd, process_creds.cmcred_pid, std::strerror(errno));
 #endif
 
 		auto ppid = process_pids.emplace(fd, process_creds.cmcred_pid).first;
@@ -593,7 +585,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 
 			auto ppid = process_pids.find(itr->fd);
 			if(debug)
-				std::cerr << "fd " << itr->fd << '(' << (ppid == process_pids.end() ? -1 : ppid->second) << "): has " << itr->revents << '\n';
+				std::fprintf(stderr, "fd %d(%d): has %d\n", itr->fd, ppid == process_pids.end() ? -1 : ppid->second, itr->revents);
 
 
 #if __NetBSD__  // If peer closes, we get POLLIN instead of POLLHUP; this is known as "good design"
@@ -604,7 +596,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				else if(errno == EPIPE)
 					itr->revents |= POLLHUP;
 				else
-					std::cerr << "fd " << itr->fd << ": send(MSG_OOB): unexpected error " << std::strerror(errno) << '\n';
+					std::fprintf(stderr, "fd %d: send(MSG_OOB): unexpected error %s\n", itr->fd, std::strerror(errno));
 			}
 #endif
 			if(itr->revents & POLLHUP) {
@@ -638,7 +630,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 						memcpy(&process_creds, CMSG_DATA(cmsg), sizeof(process_creds));
 						ppid = auth_process(itr->fd, process_creds);
 					} else {
-						std::cerr << "fd " << itr->fd << ": didn't send creds!\n";
+						std::fprintf(stderr, "fd %d: didn't send creds!\n", itr->fd);
 						continue;
 					}
 
@@ -654,15 +646,15 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 
 
 				if(cnt == -1)
-					std::cerr << "fd " << itr->fd << '(' << ppid->second << "): recv: " << std::strerror(errno) << '\n';
+					std::fprintf(stderr, "fd %d(%d): recv: %s\n", itr->fd, ppid->second, std::strerror(errno));
 				else if(cnt == sizeof(febug_message)) {
 					const febug_message * fmsg = (febug_message *)buf;
 					std::string_view name{fmsg->name, sizeof(fmsg->name)};
 					if(auto pos = name.find('\0'); pos != std::string_view::npos)
 						name = name.substr(0, pos);
 					if(debug)
-						std::cerr << "fd " << itr->fd << '(' << ppid->second << "): " << fmsg->variable_id << ", " << fmsg->variable_type << ", " << strsignal(fmsg->signal)
-						          << ", " << name << '\n';
+						std::fprintf(stderr, "fd %d(%d): %" PRIu64 ", %" PRIu64 ", %s, %.*s\n", itr->fd, ppid->second, fmsg->variable_id, fmsg->variable_type,
+						             strsignal(fmsg->signal), SV(name));
 
 					{
 						const std::unique_lock p_lock{processes_mtx};
@@ -672,7 +664,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				} else if(cnt == sizeof(stop_febug_message)) {
 					const stop_febug_message * sfmsg = (stop_febug_message *)buf;
 					if(debug)
-						std::cerr << "fd " << itr->fd << '(' << ppid->second << "): stop " << sfmsg->variable_id << '\n';
+						std::fprintf(stderr, "fd %d(%d): stop %" PRIu64 "\n", itr->fd, ppid->second, sfmsg->variable_id);
 
 					{
 						const std::unique_lock p_lock{processes_mtx};
@@ -681,7 +673,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				} else if(cnt == 0)
 					;  // just auth
 				else
-					std::cerr << "fd " << itr->fd << '(' << ppid->second << ": unknown length " << cnt << '\n';
+					std::fprintf(stderr, "fd %d(%d): unknown length %zd\n", itr->fd, ppid->second, cnt);
 			}
 		}
 
@@ -693,7 +685,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				close(fd);
 				listening_to.erase(itr);
 			} else {
-				std::cerr << "fd " << fd << ": staged for removal, but doesn't exist?\n";
+				std::fprintf(stderr, "fd %d: staged for removal, but doesn't exist?\n", fd);
 				// Weird, shouldn't happen. but what can you do
 			}
 
@@ -713,7 +705,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 
 		if(listening_to[0].revents) {
 			if(debug)
-				std::cerr << "cs: " << listening_to[0].revents << '\n';
+				std::fprintf(stderr, "cs: %d\n", listening_to[0].revents);
 
 			if(int to_add = accept(cs, nullptr, nullptr); to_add != -1) {
 #if __linux__ || __NetBSD__
@@ -724,7 +716,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				// This is the recommended-by-unix(4) way to do this on NetBSD
 				const int yes = 1;
 				if(setsockopt(to_add, SOL_CREDS_LEVEL, SO_PASSCRED, &yes, sizeof(yes)) == -1)
-					std::cerr << "fd " << to_add << ": setsockopt(SO_PASSCRED): " << std::strerror(errno) << '\n';
+					std::fprintf(stderr, "fd %d: setsockopt(SO_PASSCRED): %s\n", to_add, std::strerror(errno));
 #endif
 #if __OpenBSD__ || __APPLE__
 				cmsgcred process_creds{};
@@ -734,7 +726,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 #else
 				socklen_t opt_len = sizeof(pid_t);
 				if(!(getsockopt(to_add, SOL_LOCAL, LOCAL_PEEREPID, &process_creds.cmcred_pid, &opt_len) != -1 && opt_len == sizeof(pid_t))) {
-					std::cerr << "fd " << to_add << ": getsockopt(LOCAL_PEEREPID): " << std::strerror(errno) << ", dropping" << '\n';
+					std::fprintf(stderr, "fd %d: getsockopt(LOCAL_PEEREPID): %s, dropping\n", to_add, std::strerror(errno));
 					close(to_add);
 					continue;
 				}
@@ -745,7 +737,7 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 #endif
 					auth_process(to_add, process_creds);
 				else {
-					std::cerr << "fd " << to_add << ": getsockopt(SO_PEERCRED): " << std::strerror(errno) << ", dropping" << '\n';
+					std::fprintf(stderr, "fd %d: getsockopt(SO_PEERCRED): %s, dropping\n", to_add, std::strerror(errno));
 					close(to_add);
 					continue;
 				}
@@ -760,15 +752,17 @@ void process_handler(int cs, std::atomic<bool> * keep_going) {
 				listening_to.emplace(std::upper_bound(listening_to.begin() + 1, listening_to.end(), to_add, [](int lhs, const pollfd & rhs) { return lhs < rhs.fd; }),
 				                     pollfd{to_add, POLLIN, 0});
 			} else
-				std::cerr << "accept: " << std::strerror(errno) << '\n';
+				std::fprintf(stderr, "accept: %s\n", std::strerror(errno));
 		}
 	}
 
-	std::cerr << "poll: " << std::strerror(errno) << '\n';
+	std::fprintf(stderr, "poll: %s\n", std::strerror(errno));
 }
 
 
-enum fuse_tag {};
+struct fuse_tag {
+	inline operator const char *() const { __builtin_unreachable(); }
+};
 template <class = void>
 fuse_tag fuse_package_version();
 template <class = void>
@@ -781,47 +775,50 @@ int main(int argc, char ** argv) {
 
 	std::vector<const char *> true_argv(argv, argv + argc);
 
-	if(std::any_of(argv + 1, argv + argc, [](std::string_view arg) { return arg == "-h" || arg == "--help"; })) {
-		std::cout << "usage: " << *argv
-		          << " [-h|--help] [-v|--version] [-d] [FUSE options] <mountpoint>\n"
+	if(std::any_of(argv + 1, argv + argc, [](const std::string_view & arg) { return arg == "-h"sv || arg == "--help"sv; })) {
+		std::fprintf(stdout,
+		             "usage: %s [-h|--help] [-V|--version] [-d] [FUSE options] <mountpoint>\n"
 		             "\n"
-		             "FUSE options (-f"
-		          << ((geteuid() == 0) ? ", -o default_permissions, and -o allow_other" : " and -o default_permissions") << " are set by default):\n";
+		             "FUSE options (-f%s are set by default):\n",
+		             *argv, (geteuid() == 0) ? ", -o default_permissions, and -o allow_other" : " and -o default_permissions");
 		if constexpr(std::is_same_v<decltype(fuse_lib_help(nullptr)), fuse_tag>) {
 			if constexpr(std::is_same_v<decltype(fuse_cmdline_help()), fuse_tag>)
-				std::cout << "Stingy libfuse, none known!\n"
-				             "Assuming at least:\n"
-				             "\t-f                      run in foreground\n"
-				             "\t-o default_permissions  check file permissions in kernel\n"
-				             "\t-o allow_other          allow access by all users\n"
-				             "\n";
+				std::fputs("Stingy libfuse, none known!\n"
+				           "Assuming at least:\n"
+				           "\t-f                      run in foreground\n"
+				           "\t-o default_permissions  check file permissions in kernel\n"
+				           "\t-o allow_other          allow access by all users\n"
+				           "\n",
+				           stdout);
 			else
 				fuse_cmdline_help();
 		} else {
 			fuse_args fa{argc, argv, false};
 			fuse_lib_help(&fa);
 		}
-		std::cout << "\n"
+		std::fprintf(stdout,
+		             "\n"
 		             "Environment variables:\n"
-		             "\tFEBUG_SOCKET=["
-		          << DEFAULT_SOCK << "]\n";
+		             "\tFEBUG_SOCKET=[%s]\n",
+		             DEFAULT_SOCK);
 		return 0;
 	}
 
-	if(std::any_of(argv + 1, argv + argc, [](std::string_view arg) { return arg == "-V" || arg == "--version"; })) {
-		std::cout << "febug " << FEBUG_VERSION
-		          << "\n"
-		             "FUSE ";
+	if(std::any_of(argv + 1, argv + argc, [](const std::string_view & arg) { return arg == "-V"sv || arg == "--version"sv; })) {
+		std::fprintf(stdout,
+		             "febug %s\n"
+		             "FUSE ",
+		             FEBUG_VERSION);
 		if constexpr(std::is_same_v<decltype(fuse_package_version()), fuse_tag>) {
 			const auto ver = fuse_version();
-			std::cout << (ver / 10) << '.' << (ver % 10);
+			std::fprintf(stdout, "%d.%d", ver / 10, ver % 10);
 		} else
-			std::cout << fuse_package_version();
-		std::cout << '\n';
+			std::fputs((const char *)fuse_package_version(), stdout);
+		std::fputc('\n', stdout);
 		return 0;
 	}
 
-	if(std::find(argv + 1, argv + argc, std::string_view{"-d"}) != argv + argc) {
+	if(std::find(argv + 1, argv + argc, "-d"sv) != argv + argc) {
 		true_argv.emplace_back("-d");
 		debug = true;
 	}
