@@ -7,7 +7,9 @@ ThePhD
 .It
 Embark Studios
 .It
-Jasper Bekkers
+Lars Strojny
+.It
+EvModder
 .El
 .
 .Sh REPORTING BUGS
