@@ -10,4 +10,7 @@ must be in place.
 It's a no-op if
 .Va FEBUG_GLOBAL_CONTROLLED_SOCKET
 is
-.Sy -1 .
+#ifndef FEBUG_GLOBAL_CONTROLLED_SOCKET_MINUS_ONE
+#define FEBUG_GLOBAL_CONTROLLED_SOCKET_MINUS_ONE -1
+#endif
+.Sy FEBUG_GLOBAL_CONTROLLED_SOCKET_MINUS_ONE .
