@@ -41,15 +41,4 @@ Defaults to
 #endif
 .El
 .Pp
-There are two environment variables that allow a user to customise its behaviour:
-.Bl -tag -compact -offset indent -width "FEBUG_SOCKET"
-.It Ev FEBUG_DONT
-If set, don't try to connect to
-.Xr febug 8 ,
-so all library functions become no-ops.
-.It Ev FEBUG_SOCKET
-If set, use its value instead of the built-in
-.DV FEBUG_SOCKET
-to connect to
-.Xr febug 8 .
-.El
+#include "libfebug-options-env.h"
